# Copyright 2004, @Last Software, Inc.
# The code related to click 3 points was reused here from an original
# example titled rectangle.rb by @Last Software, Inc. 
# To it I added the joist code. cheers! Tim 
# Name:           JoistTool
# Description:    Create a string of joists @ centers 
# Usage:          click 3 points, create a bunch of joists contained within a defined area
# Date:           2004,10,27
# Type:           Tool
# Revised:        2004,10,27Problem with rotated rectanglular areas persists
#------------------------------------------------------------------------------------------
require 'sketchup.rb'

class JoistTool

def initialize
    $ip = Sketchup::InputPoint.new
    $ip1 = Sketchup::InputPoint.new
    reset

    # sets the default Joist settings
    $jthick = 4.cm if not $jthick
    $jdepth = 25.cm if not $jdepth 
    $joc = 50.cm if not $joc


     # Dialog box
    prompts = ["ĺ   ", "߶   ", "ȣ   "]
    values = [$jthick, $jdepth, $joc]
    results = inputbox prompts, values, "Joist parameters"

    return if not results
    $jthick, $jdepth, $joc = results
end

def reset
    $pts = []
    $state = 0
    $ip1.clear
    $drawn = false
    Sketchup::set_status_text "", SB_VCB_LABEL
    Sketchup::set_status_text "", SB_VCB_VALUE
    Sketchup::set_status_text "Click anywhere to start"
    $shift_down_time = Time.now
end

def activate
    self.reset
end

def deactivate(view)
    view.invalidate if $drawn
end

def set_current_point(x, y, view)
    if( !$ip.pick(view, x, y, $ip1) )
        return false
    end
    need_draw = true
    
    # Set the tooltip that will be displayed
    view.tooltip = $ip.tooltip
        
    # Compute points
    case $state
    when 0
        $pts[0] = $ip.position
        $pts[4] = $pts[0]
        need_draw = $ip.display? || $drawn
    when 1
        $pts[1] = $ip.position
        $length = $pts[0].distance $pts[1]
        Sketchup::set_status_text $length.to_s, SB_VCB_VALUE
    when 2
        pt1 = $ip.position
        pt2 = pt1.project_to_line $pts
        vec = pt1 - pt2

        $pt1 = $ip.position
        $pt2 = pt1.project_to_line $pts
        $vec = $pt1 - $pt2


        $width = vec.length
        if( $width > 0 )
            # test for a square
            square_point = pt2.offset(vec, $length)
            if( view.pick_helper.test_point(square_point, x, y) )
                $width = $length
                $pts[2] = $pts[1].offset(vec, $width)
                $pts[3] = $pts[0].offset(vec, $width)
                view.tooltip = "Square"
            else
                $pts[2] = $pts[1].offset(vec)
                $pts[3] = $pts[0].offset(vec)
            end
        else
            $pts[2] = $pts[1]
            $pts[3] = $pts[0]
        end
        Sketchup::set_status_text $width.to_s, SB_VCB_VALUE
    end

    view.invalidate if need_draw
end

def onMouseMove(flags, x, y, view)
    self.set_current_point(x, y, view)
end

# creates a new point from p1 in the direction of p2-p3 with length d
# params are Point3d, 2 vertices, a length, returns a Point3d
def translate(p1, p2, p3, d)
        v = p3 - p2
        v.length = d
        trans=Geom::Transformation.translation(v)
        return p1.transform(trans)
end   
    
def create_rectangle
#--------------------------------------------------------------------------
    model=Sketchup.active_model
    model.start_operation "Create Joists"
    entities=model.active_entities
    
    
    
          
    #rectangular plan area containing joists
    #THE FOLLOWING LINE IS FOR TESTING PURPOSES ONLY
    base=entities.add_face($pts[0], $pts[1], $pts[2], $pts[3])
    #base.pushpull $jdepth
        
    #------create a new set of points from the original 3 point pick
    $pt0=Geom::Point3d.new($pts[0][0], $pts[0][1], $pts[0][2])
    $pt1=Geom::Point3d.new($pts[1][0], $pts[1][1], $pts[1][2])
    $pt2=Geom::Point3d.new($pts[2][0], $pts[2][1], $pts[2][2])
    $pt3=Geom::Point3d.new($pts[3][0], $pts[3][1], $pts[3][2])
    
    #------find the points for the first joist thickness btw. $pt0 & #pt3
    $pt0j=translate($pt0, $pt0, $pt3, $jthick)
    #------find the points for the first joist thickness btw. $pt1 & #pt2
    $pt1j=translate($pt1, $pt1, $pt2, $jthick)
    
    #------get the number of joists required for the related $width/$joc
    #------$joc = joist on center spacing    
    $num_joists=(($width/$joc)+1).to_i 
    
    definitions=model.definitions
    count=definitions.add
    entities=count.entities
        
    #------Do first joist 
    base=entities.add_face($pt0, $pt0j, $pt1j, $pt1)
    base.reverse!
    base.pushpull $jdepth
    
    #------Do transform of location  
    t=Geom::Transformation.translation(Geom::Vector3d.new(0, 0, 0))
    entities=model.active_entities
    
    entities.add_instance(count, t)
    
    #------Copy joists to their new locations
    i = 1
    while i < $num_joists
       # Transformation
       i = i + 1
       #t=translate($pt0, $pt0, $pt3, $joc*(i-1))
       #t=translate($pt1, $pt1, $pt2, $joc*(i-1))
       vec = $pt3 - $pt0
       #vec.length = $pt0.distance $pt3
       vec.length = $joc*(i-1)
       #t=Geom::Transformation.translation(Geom::Vector3d.new(0,$joc*(i-1), 0))
       t=Geom::Transformation.translation(vec)
       entities.add_instance(count, t)
    end
        
    #------find the points for last joist thickness btw. $pt0 & #pt3
    $pt2j=translate($pt2, $pt2, $pt1, $jthick)
    #------find the points for last joist thickness btw. $pt1 & #pt2
    $pt3j=translate($pt3, $pt3, $pt0, $jthick)
    
    #------Do the last joist 
    group=entities.add_group
    entities=group.entities
    base=entities.add_face($pt2, $pt2j, $pt3j, $pt3)
    base.reverse!
    base.pushpull $jdepth
        
    self.reset
    model.commit_operation
end

def increment_state
    $state += 1
    case $state
    when 1
        $ip1.copy! $ip
        Sketchup::set_status_text "Click for second point"
        Sketchup::set_status_text "Width", SB_VCB_LABEL
        Sketchup::set_status_text "", SB_VCB_VALUE
    when 2
        $ip1.clear
        Sketchup::set_status_text "Click for third point"
        Sketchup::set_status_text "Height", SB_VCB_LABEL
        Sketchup::set_status_text "", SB_VCB_VALUE
    when 3
        self.create_rectangle
    end
end

def onLButtonDown(flags, x, y, view)
    self.set_current_point(x, y, view)
    self.increment_state
    view.lock_inference
end

def onCancel(flag, view)
    view.invalidate if $drawn
    self.reset
end

# This is called when the user types a value into the VCB
def onUserText(text, view)
    # The user may type in something that we can't parse as a length
    # so we set up some exception handling to trap that
    begin
        value = text.to_l
    rescue
        # Error parsing the text
        UI.beep
        value = nil
        Sketchup::set_status_text "", SB_VCB_VALUE
    end
    return if !value
    
    case $state
    when 1
        # update the width
        vec = $pts[1] - $pts[0]
        if( vec.length > 0.0 )
            vec.length = value
            $pts[1] = $pts[0].offset(vec)
            view.invalidate
            self.increment_state
        end
    when 2
        # update the height
        vec = $pts[3] - $pts[0]
        if( vec.length > 0.0 )
            vec.length = value
            $pts[2] = $pts[1].offset(vec)
            $pts[3] = $pts[0].offset(vec)
            self.increment_state
        end
    end
end

def getExtents
    bb = Geom::BoundingBox.new
    case $state
    when 0
        # We are getting the first point
        if( $ip.valid? && $ip.display? )
            bb.add $ip.position
        end
    when 1
        bb.add $pts[0]
        bb.add $pts[1]
    when 2
        bb.add $pts
    end
    bb
end

def draw(view)
    $drawn = false
    
    # Show the current input point
    if( $ip.valid? && $ip.display? )
        $ip.draw(view)
        $drawn = true
    end

    # show the rectangle
    if( $state == 1 )
        # just draw a line from the start to the end point
        view.set_color_from_line($ip1, $ip)
        inference_locked = view.inference_locked?
        view.line_width = 3 if inference_locked
        view.draw(GL_LINE_STRIP, $pts[0], $pts[1])
        view.line_width = 1 if inference_locked
        $drawn = true
    elsif( $state > 1 )
        # draw the curve
        view.drawing_color = "black"
        view.draw(GL_LINE_STRIP, $pts)
        $drawn = true
    end
end

def onKeyDown(key, rpt, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY && rpt == 1 )
        $shift_down_time = Time.now
        
        # if we already have an inference lock, then unlock it
        if( view.inference_locked? )
            view.lock_inference
        elsif( $state == 0 )
            view.lock_inference $ip
        elsif( $state == 1 )
            view.lock_inference $ip, $ip1
        end
    end
end

def onKeyUp(key, rpt, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY &&
        view.inference_locked? &&
        (Time.now - $shift_down_time) > 0.5 )
        view.lock_inference
    end
end

  
end # of class JoistTool
#=============================================================================
def JoistTool
    Sketchup.active_model.select_tool JoistTool.new
end

#if( not file_loaded?(".rb") )
     add_separator_to_menu("Tools")
     utilities_menu = UI.menu("Tools").add_item("   ") { Sketchup.active_model.select_tool JoistTool.new }
#end
#-----------------------------------------------------------------------------
file_loaded("JoistTool.rb")
