# Copyright 2005, Rick Wilson 

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

# Name :          pathCopy
# Description :   Copies a selected group/component along a selected path
# Author :        Rick Wilson
# Usage :         1. Install into the plugins directory (or subfolder - supports organize.rb) or into the 
#                    Plugins/examples directory and manually load from the ruby console "load 'examples/bomb.rb'" 
#                 2. Run "Copy along path" from the Plugins menu(or organized submenu).
# Date :          2005-12-05
# Type :          Tool
# History:
#			2.0 (2005-12-05) - planned improvements:
#							Specify distance between copies
#							Specify divisor
#			1.0 (2005-11-29) - first version - copies selected component to each vertex of a curve
# 

require 'sketchup.rb'
require 'vector.flat_angle.rb'

class PathCopy
	@@nCursor = 0
	@@type="Node"
#	@@added_items=[]
	@@count=0

def initialize
	@path=nil
	@group=nil
	@state=0
	if $RW_path_copy_spacing
		@spacing=$RW_path_copy_spacing
	else
		@spacing=120.inch
	end
	if(@@nCursor == 0)
		path = Sketchup.find_support_file "pathcopyCursor.png", "Plugins/Icons"
		@@nCursor = UI::create_cursor(path,0,30) if path
	end
end

def activate
	if @@type=="Spacing"
		Sketchup::set_status_text("Distance between:", SB_VCB_LABEL)
		Sketchup::set_status_text(@spacing, SB_VCB_VALUE)
	else
		Sketchup::set_status_text("Copy to Vertices", SB_VCB_LABEL)
	end
	puts "activating..."
	sel=Sketchup.active_model.selection
	if sel.length>0
		if sel.first.typename=="Edge"
			if sel.first.curve
				@state=1
				@path=sel.first.curve
				puts "path selected"
			else
				@state=1
				@path=sel.first
				puts "path selected"
			end
		end
	    Sketchup::set_status_text($exStrings.GetString("Select group/component to copy"), SB_PROMPT)
	else
	    Sketchup::set_status_text($exStrings.GetString("Select path"), SB_PROMPT)
	end
end

def onSetCursor()
	cursor = UI::set_cursor(@@nCursor)
end

def onLButtonDown(flags,x,y,view)
	if @state>1
		begin
			Sketchup.active_model.commit_operation
		rescue
			puts "nothing to commit"
		end
		@state=0
		@path=nil
#		@group=nil
	end
	ph=view.pick_helper
	num=ph.do_pick x,y
	item=ph.best_picked
#	puts item

	if item
		if @path==nil
			get_path(item) 
		else
			if item.typename=="ComponentInstance"
				points=@path.vertices
#				Sketchup.active_model.start_operation "pathCopy"
				points.collect!{|p| p.position}
				rotation=[]
				points.each {|e| rotation<<0}
				pathCopyComponent(item,points,rotation)
#				Sketchup.active_model.commit_operation
			elsif item.typename=="Group"
				points=@path.vertices
#				Sketchup.active_model.start_operation "pathCopy"
				points.collect!{|p| p.position}
				rotation=[]
				points.each {|e| rotation<<0}
				pathCopyGroup(item,points,rotation)
#				Sketchup.active_model.commit_operation
			end
		end
	end
end

def get_path(item)
	if item
		if item.typename=="Edge"
			if item.curve
				@state=1
				@path=item.curve
				puts "path selected"
			else
				@state=1
				@path=item
				puts "path selected"
			end
			Sketchup::set_status_text($exStrings.GetString("Select group/component to copy"), SB_PROMPT)
		end
	end
	Sketchup.active_model.selection.clear
	Sketchup.active_model.selection.add(@path.edges) if @path.typename=="Curve"|| @path.typename=="ArcCurve"
	Sketchup.active_model.selection.add(@path) if @path.typename=="Edge"
end

def pathCopyComponent(item,points,rotation)
#	@@added_items=[]
	@@count+=1
	Sketchup.active_model.selection.clear
	model=Sketchup.active_model
	ents=model.active_entities
	puts "Copy component instance along path"
#	points=@path.vertices
#	points.collect!{|p| p.position}
#	model.start_operation "pathCopyComponent #{@@count}"
	model.start_operation "pathCopyComponent"
	0.upto(points.length-1) do |i|
		t=Geom::Transformation.new(points[i])
		tr=Geom::Transformation.rotation(points[i],Geom::Vector3d.new(0,0,1),rotation[i])
#		@@added_items<<ents.add_instance(item.definition,t).transform!(tr)
		ents.add_instance(item.definition,t).transform!(tr)
	end
	model.commit_operation
	@state=2
end

def pathCopyGroup(item,points,rotation)
#	@@added_items=[]
	@@count+=1
	Sketchup.active_model.selection.clear
	model=Sketchup.active_model
	ents=model.active_entities
	puts "Copy group along path"
#	points=@path.vertices
#	points.collect!{|p| p.position}
#	model.start_operation "pathCopyGroup #{@@count}"
	model.start_operation "pathCopyGroup"
	0.upto(points.length-1) do |i|
		t1=Geom::Point3d.new(0,0,0)-item.transformation.origin
		t2=Geom::Transformation.new(points[i])
		tr=Geom::Transformation.rotation(points[i],Geom::Vector3d.new(0,0,1),rotation[i])
		gp=item.copy
		gp.transformation=item.transformation
		gp.transform!(t1)
		gp.transform!(t2)
		gp.transform!(tr)
#		@@added_items<<gp
	end
	model.commit_operation
	@state=2
end

def deactivate(view)
#	begin
#		Sketchup.active_model.commit_operation
#	rescue
#		puts "nothing to commit"
#	end
	view.invalidate if @drawn
end

def onCancel(flag, view)
	self.reset(view)
end

def reset(view)
#	begin
#		Sketchup.active_model.commit_operation
#	rescue
#		puts "nothing to commit"
#	end
	@path=nil
	Sketchup::set_status_text($exStrings.GetString("Select path"), SB_PROMPT)
	if( view )
		view.tooltip = nil
		view.invalidate
	end
	
end

end #class PathCopy

class PathCopySpacing < PathCopy
	@@type="Spacing"
	@@count=0

def onLButtonDown(flags,x,y,view)
#	puts "@state is #{@state}"
	if @state>1
#		begin
#			Sketchup.active_model.commit_operation
#		rescue
#			puts "nothing to commit"
#		end
		@state=0
		@path=nil
		@group=nil
		@@count=0
	end
	ph=view.pick_helper
	num=ph.do_pick x,y
	item=ph.best_picked
#	puts item

	if item
		if @path==nil
			get_path(item)
			@state=1
		else
			@group=item
			if @group.typename=="ComponentInstance"
				points,rotation=getPoints(@path)
#				Sketchup.active_model.start_operation "pathCopy #{@@count}"
				rotation=[]
				points.each {|e| rotation<<0}
				pathCopyComponent(item,points,rotation)
#				Sketchup.active_model.commit_operation
			elsif @group.typename=="Group"
				points,rotation=getPoints(@path)
#				Sketchup.active_model.start_operation "pathCopy #{@@count}"
				rotation=[]
				points.each {|e| rotation<<0}
				pathCopyGroup(item,points,rotation)
#				Sketchup.active_model.commit_operation
			end
		end
	end
end

def onUserText(text, view)
	begin
		value = text.to_l
	rescue
		# Error parsing the text
		UI.beep
		puts "Cannot convert #{text} to a Length"
		value = nil
		Sketchup::set_status_text @spacing, SB_VCB_VALUE
	end
	return if !value

	@spacing=value
	if @state==2
		@@count+=1
#		Sketchup.active_model.start_operation "erase copies"
#		@@added_items.each {|e| e.erase!}
		Sketchup.undo
#		Sketchup.active_model.start_operation "pathCopyRedo #{@@count}"
		points,rotation=getPoints(@path)
		pathCopyComponent(@group,points,rotation) if @group.typename=="ComponentInstance"
		pathCopyGroup(@group,points,rotation) if @group.typename=="Group"
#		Sketchup.active_model.commit_operation
	end
	$RW_path_copy_spacing=@spacing
end

def getPoints(path)
	puts "Getting points from path"
	runningLength=@spacing
	points=[]
	rotation=[]
	points<<path.edges.first.start.position
	rotation<<path.edges.first.line[1].flat_angle
	path.edges.each do |edge|
 #		puts edge
		if runningLength>edge.length
			runningLength-=edge.length
		#	puts "edge is only #{edge.length.to_f} long, runningLength is now #{runningLength}"
		elsif runningLength==edge.length
		#	puts "edge exactly right, adding point #{edge.end.position}"
			points<<edge.end.position
			rotation<<edge.line[1].flat_angle
			runningLength=@spacing
		else
		#	puts "edge is #{edge.length.to_f}, longer than runningLength #{runningLength}."
			vec=edge.end.position-edge.start.position
			vec.length=runningLength
			points<<edge.start.position+vec
			rotation<<edge.line[1].flat_angle
			runningLength=@spacing-(edge.start.position+vec).distance(edge.end.position)
		#	puts "subtracting #{(edge.start.position+vec).distance(edge.end.position).to_f} from edge, runningLength is now #{runningLength}"
			if runningLength<0
				while runningLength<0
					vec.length=@spacing
					points<<points.last+vec
					rotation<<edge.line[1].flat_angle
					runningLength=@spacing-points.last.distance(edge.end.position)
				#	puts "remaining edge length is #{points.last.distance(edge.end.position).to_f}"
				#	puts "runningLength #{runningLength}."
				#	return if UI.messagebox("Continue?",MB_YESNO)==7
				end
			end
		end
	end
#	puts points
	return points,rotation
end

end #class PathCopySpacing

if( not file_loaded?("pathcopy.rb") )
# BEGIN CHANGES BY organizerEdit.rb
if $submenu!=nil
	submenu=$submenu.add_submenu("·")
		submenu.add_item("·ڵ㸴") { Sketchup.active_model.select_tool PathCopy.new }
		submenu.add_item("ָ븴") { Sketchup.active_model.select_tool PathCopySpacing.new }
else
	submenu=UI.menu("Plugins").add_submenu("·")
		submenu.add_item("·ڵ㸴") { Sketchup.active_model.select_tool PathCopy.new }
		submenu.add_item("ָ븴") { Sketchup.active_model.select_tool PathCopySpacing.new }
end
# END CHANGES BY organizerEdit.rb
end
#-----------------------------------------------------------------------------
file_loaded("pathcopy.rb")
